@@ -131,7 +131,7 @@ gda_xslt_bk_section (GdaXsltExCont * exec, GdaXsltIntCont * pdata,
 	}
 	if (query_node == NULL) {
 		g_set_error (&(exec->error), 0, 0,
-			     "no query node in section node");
+			     "%s", "no query node in section node");
 		return -1;
 	}
 	/* set the query context */
@@ -396,7 +396,7 @@ _utility_data_model_to_nodeset (GdaDataModel * model,
 #ifdef GDA_DEBUG_NO
 		g_print ("xmlNewNode return NULL\n");
 #endif
-		g_set_error (error, 0, 0, "xmlNewNode return NULL\n");
+		g_set_error (error, 0, 0, "%s", "xmlNewNode return NULL\n");
 		return -1;
 	}
 	/* compute columns */
@@ -423,8 +423,7 @@ _utility_data_model_to_nodeset (GdaDataModel * model,
 		xmlNodePtr row, field;
 		gint r, c;
 		for (r = 0; r < nrows; r++) {
-			row = xmlNewChild (mainnode, NULL,
-					   (xmlChar *) "row", NULL);
+			row = xmlNewChild (mainnode, NULL, (xmlChar *) "row", NULL);
 			for (c = 0; c < rnb_cols; c++) {
 				GValue *value;
 				xmlChar *str = NULL;
@@ -435,13 +434,14 @@ _utility_data_model_to_nodeset (GdaDataModel * model,
 					for (c = 0; c < rnb_cols; c++)
 						g_free (col_ids[c]);
 					g_free (col_ids);
+					g_free (rcols);
 					return -1;
 				}
 				if (gda_value_is_null ((GValue *) value))
 					isnull = TRUE;
 				else 
 					str = value_to_xmlchar (value);
-				field = xmlNewChild (row, NULL, (xmlChar *) "column", (xmlChar *) str);
+				field = xmlNewTextChild (row, NULL, (xmlChar *) "column", (xmlChar *) str);
 				xmlSetProp (field, (xmlChar *) "name", (xmlChar *) col_ids[c]);
 				if (isnull)
 					xmlSetProp (field, (xmlChar *) "isnull", (xmlChar *) "true");
@@ -452,8 +452,10 @@ _utility_data_model_to_nodeset (GdaDataModel * model,
 	for (c = 0; c < rnb_cols; c++)
 		g_free (col_ids[c]);
 	g_free (col_ids);
+	g_free (rcols);
 
 	*nodeset = (xmlXPathObjectPtr) xmlXPathNewNodeSet (mainnode);
+
 	return 0;
 }
 
@@ -504,7 +506,7 @@ gda_xslt_bk_internal_query (GdaXsltExCont * exec, GdaXsltIntCont * pdata,
 	query_name = xmlGetProp (query_node, (const xmlChar *) "name");
 	if (query_name == NULL) {
 		g_set_error (&(exec->error), 0, 0,
-			     "the query element is not correct, no 'name' attribute\n");
+			     "%s", "the query element is not correct, no 'name' attribute\n");
 		return -1;
 	}
 
@@ -515,7 +517,7 @@ gda_xslt_bk_internal_query (GdaXsltExCont * exec, GdaXsltIntCont * pdata,
 		sqltxt_node = query_node->children;
 		if (sqltxt_node == NULL || sqltxt_node->type != XML_TEXT_NODE) {
 			g_set_error (&(exec->error), 0, 0,
-				     "the query element is not correct, it have not a first text children\n");
+				     "%s", "the query element is not correct, it have not a first text children\n");
 			return -1;
 		}
 #ifdef GDA_DEBUG_NO
@@ -651,6 +653,10 @@ value_to_xmlchar (const GValue * value)
 		g_hash_table_insert (data_handlers, (gpointer) G_TYPE_UCHAR,
 				     gda_handler_numerical_new ());
 		g_hash_table_insert (data_handlers, (gpointer) G_TYPE_ULONG,
+				     gda_handler_numerical_new ());
+		g_hash_table_insert (data_handlers, (gpointer) G_TYPE_LONG,
+				     gda_handler_numerical_new ());
+		g_hash_table_insert (data_handlers, (gpointer) G_TYPE_GTYPE,
 				     gda_handler_type_new ());
 		g_hash_table_insert (data_handlers, (gpointer) G_TYPE_UINT,
 				     gda_handler_numerical_new ());
