@@ -1,6 +1,6 @@
 /* gda-data-model-iter.c
  *
- * Copyright (C) 2005 - 2008 Vivien Malerba
+ * Copyright (C) 2005 - 2010 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -276,6 +276,12 @@ validate_holder_change_cb (GdaSet *paramlist, GdaHolder *param, const GValue *ne
 	GdaDataModelIter *iter;
 	gint col;
 	GError *error = NULL;
+	GValue *nvalue;
+
+	if (new_value)
+		nvalue = (GValue*) new_value;
+	else
+		nvalue = gda_value_new_null();
 
 	iter = (GdaDataModelIter *) paramlist;
 	if (!iter->priv->keep_param_changes && (iter->priv->row >= 0)) {
@@ -289,14 +295,14 @@ validate_holder_change_cb (GdaSet *paramlist, GdaHolder *param, const GValue *ne
 				     _("Column %d out of range (0-%d)"), col, g_slist_length (paramlist->holders) - 1);
 		else if (GDA_DATA_MODEL_GET_CLASS ((GdaDataModel *) iter->priv->data_model)->i_iter_set_value) {
 			if (! (GDA_DATA_MODEL_GET_CLASS ((GdaDataModel *) iter->priv->data_model)->i_iter_set_value) 
-			    ((GdaDataModel *) iter->priv->data_model, iter, col, new_value, &error)) {
+			    ((GdaDataModel *) iter->priv->data_model, iter, col, nvalue, &error)) {
 				if (!error)
 					g_set_error (&error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
 						      "%s", _("GdaDataModel refused value change"));
 			}
 		}
 		else if (! gda_data_model_set_value_at ((GdaDataModel *) iter->priv->data_model, 
-							col, iter->priv->row, new_value, &error)) {
+							col, iter->priv->row, nvalue, &error)) {
 			if (!error)
 				g_set_error (&error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
 					      "%s", _("GdaDataModel refused value change"));
@@ -305,6 +311,8 @@ validate_holder_change_cb (GdaSet *paramlist, GdaHolder *param, const GValue *ne
 		g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [0]);
 		g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [1]);
 	}
+	if (!new_value)
+		gda_value_free (nvalue);
 
 	if (!error && ((GdaSetClass *) parent_class)->validate_holder_change)
 		/* for the parent class */
@@ -437,16 +445,42 @@ gda_data_model_iter_set_property (GObject *object,
 				}
 				else {
 					const gchar *cstr;
+					gchar *tmp;
+
+					/* ID attribute */
 					cstr = gda_column_get_description (column);
 					if (!cstr)
 						cstr = gda_column_get_name (column);
 					if (cstr)
-						g_object_set (G_OBJECT (param), "id", cstr, NULL);
-					else {
-						gchar *tmp = g_strdup_printf ("col%d", col);
-						g_object_set (G_OBJECT (param), "id", tmp, NULL);
-						g_free (tmp);
+						tmp = (gchar *) cstr;
+					else
+						tmp = g_strdup_printf ("col%d", col);
+
+					if (gda_set_get_holder ((GdaSet *) iter, tmp)) {
+						gint e;
+						for (e = 1; ; e++) {
+							gchar *tmp2 = g_strdup_printf ("%s_%d", tmp, e);
+							if (! gda_set_get_holder ((GdaSet *) iter, tmp2)) {
+								g_object_set (G_OBJECT (param), "id", tmp2, NULL);
+								g_free (tmp2);
+								break;
+							}
+							g_free (tmp2);
+						}
 					}
+					else
+						g_object_set (G_OBJECT (param), "id", tmp, NULL);
+
+					if (!cstr)
+						g_free (tmp);
+
+					/* other attributes */
+					cstr = gda_column_get_description (column);
+					if (cstr)
+						g_object_set (G_OBJECT (param), "description", cstr, NULL);
+					cstr = gda_column_get_name (column);
+					if (cstr)
+						g_object_set (G_OBJECT (param), "name", cstr, NULL);
 				}
 				if (gda_column_get_default_value (column))
 					gda_holder_set_default_value (param, gda_column_get_default_value (column));
@@ -528,7 +562,7 @@ gda_data_model_iter_get_property (GObject *object,
 }
 
 /**
- * gda_data_model_iter_move_to_row
+ * gda_data_model_iter_move_to_row:
  * @iter: a #GdaDataModelIter object
  * @row: the row to set @iter to
  *
@@ -538,7 +572,8 @@ gda_data_model_iter_get_property (GObject *object,
  * property is set to -1 (which means that gda_data_model_iter_is_valid() would return FALSE)
  *
  * If any other error occurred then the returned value is FALSE, but the "current-row"
- * property is set to the @row row.
+ * property is set to the @row row.  In this case
+ * each #GdaHolder composing @iter for which an error occurred will be invalid (see gda_holder_is_valid()).
  *
  * Returns: TRUE if no error occurred
  */
@@ -561,9 +596,23 @@ gda_data_model_iter_move_to_row (GdaDataModelIter *iter, gint row)
 		GdaDataModel *model;
 		if ((gda_data_model_iter_get_row (iter) >= 0) &&
 		    (gda_data_model_iter_get_row (iter) != row) && 
-		    ! gda_set_is_valid ((GdaSet*) iter, NULL))
+		    ! _gda_set_validate ((GdaSet*) iter, NULL)) {
 			return FALSE;
-		
+		}
+
+		gboolean *null_oks = NULL;
+		if (GDA_SET (iter)->holders) {
+			gint i;
+			GSList *list;
+
+			null_oks = g_new (gboolean, g_slist_length (GDA_SET (iter)->holders));
+			for (i = 0, list = GDA_SET (iter)->holders; list; i++, list = list->next) {
+				null_oks[i] = gda_holder_get_not_null ((GdaHolder*) list->data);
+				gda_holder_set_not_null ((GdaHolder*) list->data, FALSE);
+			}
+		}
+
+		gboolean move_ok;
 		model = iter->priv->data_model;
 		if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row)
 			return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row) (model, iter, row);
@@ -587,7 +636,7 @@ set_param_attributes (GdaHolder *holder, GdaValueAttribute flags)
 }
 
 /**
- * gda_data_model_iter_move_to_row_default
+ * gda_data_model_iter_move_to_row_default: (skip)
  */
 gboolean
 gda_data_model_iter_move_to_row_default (GdaDataModel *model, GdaDataModelIter *iter, gint row)
@@ -597,6 +646,7 @@ gda_data_model_iter_move_to_row_default (GdaDataModel *model, GdaDataModelIter *
 	gint col;
 	GdaDataModel *test;
 	gboolean update_model;
+	gboolean retval = TRUE;
 	
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
 
@@ -624,21 +674,20 @@ gda_data_model_iter_move_to_row_default (GdaDataModel *model, GdaDataModelIter *
 		cvalue = gda_data_model_get_value_at (model, col, row, NULL);
 		if (!cvalue || 
 		    !gda_holder_set_value ((GdaHolder*) list->data, cvalue, NULL)) {
-			g_object_set (G_OBJECT (iter), "current-row", row, 
-				      "update-model", update_model, NULL);
-			gda_data_model_iter_invalidate_contents (iter);
-			return FALSE;
+			gda_holder_force_invalid ((GdaHolder*) list->data);
+			retval = FALSE;
 		}
-		set_param_attributes ((GdaHolder*) list->data, 
-				      gda_data_model_get_attributes_at (model, col, row));
+		else
+			set_param_attributes ((GdaHolder*) list->data, 
+					      gda_data_model_get_attributes_at (model, col, row));
 	}
 	g_object_set (G_OBJECT (iter), "current-row", row, "update-model", update_model, NULL);
-	return TRUE;
+	return retval;
 }
 
 
 /**
- * gda_data_model_iter_move_next
+ * gda_data_model_iter_move_next:
  * @iter: a #GdaDataModelIter object
  *
  * Moves @iter one row further than where it already is 
@@ -649,7 +698,8 @@ gda_data_model_iter_move_to_row_default (GdaDataModel *model, GdaDataModelIter *
  * is set to -1 (which means that gda_data_model_iter_is_valid() would return FALSE)
  *
  * If any other error occurred then the returned value is FALSE, but the "current-row"
- * property is set to the new current row (one row more than it was before the call).
+ * property is set to the new current row (one row more than it was before the call). In this case
+ * each #GdaHolder composing @iter for which an error occurred will be invalid (see gda_holder_is_valid()).
  *
  * Returns: TRUE if the iterator is now at the next row
  */
@@ -661,7 +711,7 @@ gda_data_model_iter_move_next (GdaDataModelIter *iter)
 	g_return_val_if_fail (iter->priv, FALSE);
 
 	if ((gda_data_model_iter_get_row (iter) >= 0) &&
-	    ! gda_set_is_valid ((GdaSet*) iter, NULL))
+	    ! _gda_set_validate ((GdaSet*) iter, NULL))
 		return FALSE;
 
 	model = iter->priv->data_model;
@@ -673,7 +723,7 @@ gda_data_model_iter_move_next (GdaDataModelIter *iter)
 }
 
 /**
- * gda_data_model_iter_move_next_default
+ * gda_data_model_iter_move_next_default: (skip)
  */
 gboolean
 gda_data_model_iter_move_next_default (GdaDataModel *model, GdaDataModelIter *iter)
@@ -683,6 +733,7 @@ gda_data_model_iter_move_next_default (GdaDataModel *model, GdaDataModelIter *it
 	gint row;
 	GdaDataModel *test;
 	gboolean update_model;
+	gboolean retval = TRUE;
 	
 	/* validity tests */
 	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
@@ -710,21 +761,20 @@ gda_data_model_iter_move_next_default (GdaDataModel *model, GdaDataModelIter *it
 		cvalue = gda_data_model_get_value_at (model, col, row, NULL);
 		if (!cvalue || 
 		    !gda_holder_set_value ((GdaHolder *) list->data, cvalue, NULL)) {
-			g_object_set (G_OBJECT (iter), "current-row", row, 
-				      "update-model", update_model, NULL);
-			gda_data_model_iter_invalidate_contents (iter);
-			return FALSE;
+			gda_holder_force_invalid ((GdaHolder *) list->data);
+			retval = FALSE;
 		}
-		set_param_attributes ((GdaHolder *) list->data, 
-				      gda_data_model_get_attributes_at (model, col, row));
+		else
+			set_param_attributes ((GdaHolder *) list->data, 
+					      gda_data_model_get_attributes_at (model, col, row));
 	}
 	g_object_set (G_OBJECT (iter), "current-row", row, 
 		      "update-model", update_model, NULL);
-	return TRUE;
+	return retval;
 }
 
 /**
- * gda_data_model_iter_move_prev
+ * gda_data_model_iter_move_prev:
  * @iter: a #GdaDataModelIter object
  *
  * Moves @iter one row before where it already is (synchronizes the values of the parameters in @iter 
@@ -735,7 +785,8 @@ gda_data_model_iter_move_next_default (GdaDataModel *model, GdaDataModelIter *it
  * is set to -1 (which means that gda_data_model_iter_is_valid() would return FALSE).
  *
  * If any other error occurred then the returned value is FALSE, but the "current-row"
- * property is set to the new current row (one row less than it was before the call).
+ * property is set to the new current row (one row less than it was before the call).  In this case
+ * each #GdaHolder composing @iter for which an error occurred will be invalid (see gda_holder_is_valid()).
  *
  * Returns: TRUE if the iterator is now at the previous row
  */
@@ -748,7 +799,7 @@ gda_data_model_iter_move_prev (GdaDataModelIter *iter)
 	g_return_val_if_fail (iter->priv, FALSE);
 
 	if ((gda_data_model_iter_get_row (iter) >= 0) &&
-	    ! gda_set_is_valid ((GdaSet*) iter, NULL))
+	    ! _gda_set_validate ((GdaSet*) iter, NULL))
 		return FALSE;
 
 	model = iter->priv->data_model;
@@ -760,7 +811,7 @@ gda_data_model_iter_move_prev (GdaDataModelIter *iter)
 }
 
 /**
- * gda_data_model_iter_move_prev_default
+ * gda_data_model_iter_move_prev_default: (skip)
  */
 gboolean
 gda_data_model_iter_move_prev_default (GdaDataModel *model, GdaDataModelIter *iter)
@@ -770,6 +821,7 @@ gda_data_model_iter_move_prev_default (GdaDataModel *model, GdaDataModelIter *it
 	gint row;
 	GdaDataModel *test;
 	gboolean update_model;
+	gboolean retval = TRUE;
 	
 	/* validity tests */
 	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
@@ -797,22 +849,21 @@ gda_data_model_iter_move_prev_default (GdaDataModel *model, GdaDataModelIter *it
 		cvalue = gda_data_model_get_value_at (model, col, row, NULL);
 		if (!cvalue || 
 		    !gda_holder_set_value ((GdaHolder*) list->data, cvalue, NULL)) {
-			g_object_set (G_OBJECT (iter), "current-row", row, 
-				      "update-model", update_model, NULL);
-			gda_data_model_iter_invalidate_contents (iter);
-			return FALSE;
+			gda_holder_force_invalid ((GdaHolder*) list->data);
+			retval = FALSE;
 		}
-		set_param_attributes ((GdaHolder*) list->data,
-				      gda_data_model_get_attributes_at (model, col, row));
+		else
+			set_param_attributes ((GdaHolder*) list->data,
+					      gda_data_model_get_attributes_at (model, col, row));
 	}
 	g_object_set (G_OBJECT (iter), "current-row", row, 
 		      "update-model", update_model, NULL);
-	return TRUE;
+	return retval;
 }
 
 
 /**
- * gda_data_model_iter_get_row
+ * gda_data_model_iter_get_row:
  * @iter: a #GdaDataModelIter object
  *
  * Get the row which @iter represents in the data model
@@ -829,7 +880,7 @@ gda_data_model_iter_get_row (GdaDataModelIter *iter)
 }
 
 /**
- * gda_data_model_iter_invalidate_contents
+ * gda_data_model_iter_invalidate_contents:
  * @iter: a #GdaDataModelIter object
  *
  * Declare all the parameters in @iter invalid, without modifying the
@@ -844,16 +895,13 @@ gda_data_model_iter_invalidate_contents (GdaDataModelIter *iter)
 	g_return_if_fail (iter->priv);
 
 	iter->priv->keep_param_changes = TRUE;
-	list = GDA_SET (iter)->holders;
-	while (list) {
+	for (list = GDA_SET (iter)->holders; list; list = list->next)
 		gda_holder_force_invalid (GDA_HOLDER (list->data));
-		list = g_slist_next (list);
-	}
 	iter->priv->keep_param_changes = FALSE;
 }
 
 /**
- * gda_data_model_iter_is_valid
+ * gda_data_model_iter_is_valid:
  * @iter: a #GdaDataModelIter object
  *
  * Tells if @iter is a valid iterator (if it actually corresponds to a valid row in the model)
@@ -870,7 +918,7 @@ gda_data_model_iter_is_valid (GdaDataModelIter *iter)
 }
 
 /**
- * gda_data_model_iter_get_column_for_param
+ * gda_data_model_iter_get_column_for_param:
  * @iter: a #GdaDataModelIter object
  * @param: a #GdaHolder object, listed in @iter
  *
@@ -891,14 +939,14 @@ gda_data_model_iter_get_column_for_param (GdaDataModelIter *iter, GdaHolder *par
 }
 
 /**
- * gda_data_model_iter_get_holder_for_field
+ * gda_data_model_iter_get_holder_for_field:
  * @iter: a #GdaDataModelIter object
  * @col: the requested column
  *
  * Fetch a pointer to the #GdaHolder object which is synchronized with data at 
  * column @col
  *
- * Returns: the #GdaHolder, or %NULL if an error occurred
+ * Returns: (transfer none): the #GdaHolder, or %NULL if an error occurred
  */
 GdaHolder *
 gda_data_model_iter_get_holder_for_field (GdaDataModelIter *iter, gint col)
@@ -910,13 +958,13 @@ gda_data_model_iter_get_holder_for_field (GdaDataModelIter *iter, gint col)
 }
 
 /**
- * gda_data_model_iter_get_value_at
+ * gda_data_model_iter_get_value_at:
  * @iter: a #GdaDataModelIter object
  * @col: the requested column
  *
  * Get the value stored at the column @col in @iter. The returned value must not be modified.
  *
- * Returns: the #GValue, or %NULL if the value could not be fetched
+ * Returns: (transfer none): the #GValue, or %NULL if the value could not be fetched
  */
 const GValue *
 gda_data_model_iter_get_value_at (GdaDataModelIter *iter, gint col)
@@ -934,7 +982,7 @@ gda_data_model_iter_get_value_at (GdaDataModelIter *iter, gint col)
 }
 
 /**
- * gda_data_model_iter_set_value_at
+ * gda_data_model_iter_set_value_at:
  * @iter: a #GdaDataModelIter object
  * @col: the column number
  * @value: a #GValue (not %NULL)
@@ -964,13 +1012,13 @@ gda_data_model_iter_set_value_at (GdaDataModelIter *iter, gint col, const GValue
 }
 
 /**
- * gda_data_model_iter_get_value_for_field
+ * gda_data_model_iter_get_value_for_field:
  * @iter: a #GdaDataModelIter object
  * @field_name: the requested column name
  *
  * Get the value stored at the column @field_name in @iter
  *
- * Returns: the #GValue, or %NULL
+ * Returns: (transfer none): the #GValue, or %NULL
  */
 const GValue *
 gda_data_model_iter_get_value_for_field (GdaDataModelIter *iter, const gchar *field_name)
