@@ -207,8 +207,8 @@ gda_default_escape_string (const gchar *string)
  * gda_default_unescape_string
  * @string: string to unescape
  *
- * Do the reverse of gda_default_escape_string(): transforms any "\'" into "'" and any
- * "\\" into "\". 
+ * Do the reverse of gda_default_escape_string(): transforms any "''" into "'", any
+ * "\\" into "\" and any "\'" into "'". 
  *
  * Returns: a new unescaped string, or %NULL in an error was found in @string
  */
@@ -589,6 +589,7 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 				}
 			}
 		}
+		g_free (source);
 	}
 
 	/* specified value */
@@ -1822,6 +1823,9 @@ gda_sql_identifier_quote (const gchar *id, GdaConnection *cnc, GdaServerProvider
 			prov = gda_connection_get_provider (cnc);
 	}
 
+	if ((*id == '*') && (! id [1]))
+	    return g_strdup (id);
+
 	if (prov && PROV_CLASS (prov)->identifier_quote)
 		return PROV_CLASS (prov)->identifier_quote (prov, cnc, id,
 							    for_meta_store, force_quotes);
